@@ -18,12 +18,13 @@
 # to
 # rpmbuild -bb egroupware-rh.spec                    >> $LOGFILE 2>&1
 #  
-# in the sript
+# in the script
 # How to create GPG keys to sign your rpm's you will found in a seperate
 # Document
 #
 # Script changed 2004 May 21 Reiner Jung
 # Script changed 2005 Apr 15 by Ralf Becker and Wim Bonis
+# 2005 Sep 20 Ralf Becker: disabled fedora 2 build
 
 BRANCH=Version-1_0_0-branch
 
@@ -45,8 +46,9 @@ PACKAGING=`grep "%define packaging" $SPECFILE | cut -f3 -d' '`
 PACKAGINGFEDORA=`grep "Release:" $SPECFILEFEDORA | cut -f2 -d' '`
                                                                                                                              
 HOMEBUILDDIR=`whoami`
-CVSACCOUNT=ext:ralfbecker
-#CVSACCOUNT=pserver:anonymous
+#which account to use for checkouts and updates, after that the tree is made anonymous anyway, to allow users to update
+#CVSACCOUNT=ext:ralfbecker
+CVSACCOUNT=pserver:anonymous
 ANONCVSDIR=/tmp/build_root/egroupware
 ANONCVSDIRFEDORA=/tmp/build_root/fedora
 ANONCVSDIRFEDORABUILD=/tmp/build_root/fedora/egroupware
@@ -71,23 +73,30 @@ echo "Start Build Process of - $PACKAGENAME $VERSION"
 echo "---------------------------------------"              				>> $LOGFILE 2>&1
 date                                                        				>> $LOGFILE 2>&1
 cd $ANONCVSDIR
+	
+[ "$CVSACCOUNT" = 'pserver:anonymous' ] && cvs -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware login
 
-if [ ! -d egroupware ] ; then
-
-	[ $CVSACCOUT = 'pserver:anonymous'] && cvs -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware login
-
-	cvs -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware co -r $BRANCH egroupware
+if [ ! -d egroupware/phpgwapi ]	# new checkout
+then
+	cvs -z9 -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware co -Pr $BRANCH egroupware
 	cd egroupware
-	cvs -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware co -r $BRANCH all
-
+	cvs -z9 -d:$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware co -Pr $BRANCH all
+else										# updating an existing checkout in the build-root
+	[ "$CVSACCOUNT" != 'pserver:anonymous' ] && {	# changing back to the developer account
+		echo ":$CVSACCOUNT@cvs.sourceforge.net:/cvsroot/egroupware" > Root.developer
+		find . -name CVS -exec cp Root.developer {}/Root \;
+		rm Root.developer
+	}
+	cd egroupware						# need to step into the eGW dir (no CVS dir otherwise)
+	cvs -z9 update -r $BRANCH -dP                                    		>> $LOGFILE 2>&1
 fi
+
 cd $ANONCVSDIR
 
-cvs -z9 update -r $BRANCH -dP                                     		>> $LOGFILE 2>&1
-
-echo ":pserver:anonymous@cvs.sourceforge.net:/cvsroot/egroupware" > Root.anonymous	
-find . -type d -name CVS -exec cp Root.anonymous {}/Root \;				>> $LOGFILE 2>&1
+echo ":pserver:anonymous@cvs.sourceforge.net:/cvsroot/egroupware" > Root.anonymous
+find . -name CVS -exec cp Root.anonymous {}/Root \;				>> $LOGFILE 2>&1
 rm Root.anonymous
+
 echo "End from CVS update"						     		>> $LOGFILE 2>&1
 echo "---------------------------------------"      				        >> $LOGFILE 2>&1
 find . -type d -exec chmod 775 {} \;
@@ -165,24 +174,21 @@ echo "---------------------------------------"      				        >> $LOGFILE 2>&1
 
 
 ##############################################################################################################
-#                                                                                                            # 
+#                                                                                                            #
 # Here start the build process for the Fedora packages                                                       #
 #                                                                                                            #
-############################################################################################################## 
+##############################################################################################################
 
 
-cd $ANONCVSDIRFEDORA
-tar czvf $SRCDIR/$PACKAGENAMEFEDORA-$VERSIONFEDORA.$PACKAGINGFEDORA.tar.gz egroupware   >> $LOGFILEFEDORA 2>&1
-                                                                                                                             
-                                                                                                                             
-echo "Start Build Process of - $PACKAGENAMEFEDORA $VERSIONFEDORA"                       >> $LOGFILEFEDORA 2>&1
-echo "---------------------------------------"                                          >> $LOGFILEFEDORA 2>&1
-cd $SPECDIR
-rpmbuild -ba --sign $SPECFILEFEDORA                                                     >> $LOGFILEFEDORA 2>&1
-echo "End Build Process of - $PACKAGENAMEFEDORA $VERSIONFEDORA $PACKAGINGFEDORA"        >> $LOGFILEFEDORA 2>&1
-echo "---------------------------------------"                                          >> $LOGFILEFEDORA 2>&1
-                                                                                                                             
+#cd $ANONCVSDIRFEDORA
+#tar czvf $SRCDIR/$PACKAGENAMEFEDORA-$VERSIONFEDORA.$PACKAGINGFEDORA.tar.gz egroupware   >> $LOGFILEFEDORA 2>&1
 
+#echo "Start Build Process of - $PACKAGENAMEFEDORA $VERSIONFEDORA"                       >> $LOGFILEFEDORA 2>&1
+#echo "---------------------------------------"                                          >> $LOGFILEFEDORA 2>&1
+#cd $SPECDIR
+#rpmbuild -ba --sign $SPECFILEFEDORA                                                     >> $LOGFILEFEDORA 2>&1
+#echo "End Build Process of - $PACKAGENAMEFEDORA $VERSIONFEDORA $PACKAGINGFEDORA"        >> $LOGFILEFEDORA 2>&1
+#echo "---------------------------------------"                                          >> $LOGFILEFEDORA 2>&1
 
 ##############################################################################################################
 #                                                                                                            #
@@ -196,14 +202,14 @@ echo "---------------------------------------"
 #date                                                                                    >> $LOGFILEFEBIT 2>&1
 #
 #cd $ANONCVSDIRFEDORA
-#                                                                                                                            
+# 
 #echo "build bitrock Linux package" 							>> $LOGFILEFEBIT 2>&1
 #/opt/installbuilder-2.0/bin/builder build /opt/installbuilder-2.0/projects/egroupware.xml linux
 #echo "build bitrock Windows package" 							>> $LOGFILEFEBIT 2>&1
 #/opt/installbuilder-2.0/bin/builder build /opt/installbuilder-2.0/projects/egroupware.xml windows
 
 #rm -rf egroupware
-echo "Fedora Build Root deleted $PACKAGENAMEFEDORA $VERSIONFEDORA $PACKAGINGFEDORA"     >> $LOGFILEFEBIT 2>&1
-echo "---------------------------------------"                                          >> $LOGFILEFEBIT 2>&1
+#echo "Fedora Build Root deleted $PACKAGENAMEFEDORA $VERSIONFEDORA $PACKAGINGFEDORA"     >> $LOGFILEFEBIT 2>&1
+#echo "---------------------------------------"                                          >> $LOGFILEFEBIT 2>&1
 
 
