@@ -11,7 +11,7 @@
 	*  option) any later version.                                              *
 	\**************************************************************************/
 
-	/* $Id: class.boinfolog.inc.php,v 1.51.2.5 2004/09/08 16:01:16 ralfbecker Exp $ */
+	/* $Id: class.boinfolog.inc.php,v 1.51.2.7 2005/02/14 10:30:36 ralfbecker Exp $ */
 
 	class boinfolog 			// BO: buiseness objects: internal logic
 	{
@@ -123,7 +123,7 @@
 						{
 							$this->status[$key] = array();
 						}
-						$this->status[$key] += $this->config->config_data['status'][$key];
+						$this->status[$key] = array_merge($this->status[$key],$this->config->config_data['status'][$key]);
 					}
 				}
 				if (isset($this->config->config_data['types']) && is_array($this->config->config_data['types']))
@@ -508,14 +508,14 @@
 			{
 				foreach($infos as $info)
 				{
-					$time = intval(date('Hi',$info['info_startdate']));
-					$date = date('Y/m/d',$info['info_startdate']);
+					$time = (int) adodb_date('Hi',$info['info_startdate']);
+					$date = adodb_date('Y/m/d',$info['info_startdate']);
 					if ($do_events && !$time ||
 					    !$do_events && $time && $date == $date_wanted)
 					{
 						continue;
 					}
-					$title = ($do_events?$GLOBALS['phpgw']->common->formattime(date('H',$info['info_startdate']),date('i',$info['info_startdate'])).' ':'').
+					$title = ($do_events?$GLOBALS['phpgw']->common->formattime(adodb_date('H',$info['info_startdate']),adodb_date('i',$info['info_startdate'])).' ':'').
 						$info['info_subject'];
 					$view = $this->link->view('infolog',$info['info_id']);
 					$content=array();
