@@ -11,7 +11,7 @@
 	*  option) any later version.                                              *
 	\**************************************************************************/
 
-	/* $Id: class.uiinfolog.inc.php,v 1.84.2.1 2004/08/28 17:59:21 ralfbecker Exp $ */
+	/* $Id: class.uiinfolog.inc.php,v 1.84.2.3 2005/02/24 14:10:26 ralfbecker Exp $ */
 
 	class uiinfolog 	// UI - User Interface - HTML 
 	{
@@ -431,7 +431,8 @@
 						$content['info_responsible'] = $parent['info_owner'];
 					}
 					*/
-					$content['info_status'] = 'ongoing';
+					$content['info_type'] = $parent['info_type'];
+					$content['info_status'] = $this->bo->status['defaults'][$content['info_type']];
 					$content['info_confirm'] = 'not';
 					$content['info_subject']=lang($this->messages['re']).' '.$parent['info_subject'];
 					$content['info_des'] = '';
@@ -456,6 +457,11 @@
 							{
 								$readonlys[$name] = $name != 'info_status';
 							}
+							// need to set all customfields extra, as they are not set if empty
+							foreach($this->bo->customfields as $name => $value)
+							{
+								$readonlys['#'.$name] = true;
+							}
 						}
 						else
 						{
@@ -500,6 +506,7 @@
 						{
 							$content['info_type'] = $type;
 						}
+						$content['info_status'] = $this->bo->status['defaults'][$content['info_type']];
 						break;
 				}
 				$content['link_to']['primary'] = $content['info_link_id'] ? $content['info_link_id'] : True;
