@@ -13,7 +13,7 @@
 *  option) any later version.                                              *
 \**************************************************************************/
 
-	/* $Id: class.bokb.inc.php,v 1.29.2.9 2004/10/29 16:31:19 alpeb Exp $ */
+	/* $Id: class.bokb.inc.php,v 1.29.2.10 2004/11/17 13:31:13 alpeb Exp $ */
 
 	/**
 	* Business logic layer of the Knowledge Base
@@ -21,7 +21,7 @@
 	* Last Editor:	$Author: alpeb $
 	* @author		Alejandro Pedraza
 	* @package		phpbrain
-	* @version		$Revision: 1.29.2.9 $
+	* @version		$Revision: 1.29.2.10 $
 	* @license		GPL
 	**/
 	class bokb
@@ -1269,7 +1269,7 @@
 			$GLOBALS['phpgw']->send->AddAddress($recipient);
 			$GLOBALS['phpgw']->send->Subject = get_var('subject', 'POST');
 			$GLOBALS['phpgw']->send->Body = get_var('txt_message', 'POST', lang('E-GroupWare Knowledge Base article attached'));
-			$GLOBALS['phpgw']->send->AddStringAttachment($article_contents, lang('article'), 'base64', 'text/html');
+			$GLOBALS['phpgw']->send->AddStringAttachment($article_contents, lang('article').'.html', 'base64', 'text/html');
 			$message = '';
 			if (!$GLOBALS['phpgw']->send->Send())
 			{
