@@ -18,13 +18,13 @@
   * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA            *
   \**************************************************************************/
 
-  /* $Id: class.auth_pam.inc.php,v 1.3.4.1 2005/04/15 14:48:05 ralfbecker Exp $ */
+  /* $Id: class.auth_pam.inc.php,v 1.3.4.2 2005/05/24 13:01:43 petere78 Exp $ */
 
 	class auth_
 	{
 		function authenticate($username, $passwd)
 		{
-			if (pam_auth($username, get_magic_quotes_gpc() ? stripslashes($passwd) : $passwd, $error)) 
+			if (pam_auth($username, get_magic_quotes_gpc() ? stripslashes($passwd) : $passwd, &$error)) 
 			{
 				return True;
 			}
