@@ -20,7 +20,7 @@
   * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA            *
   \**************************************************************************/
 
-  /* $Id: class.contacts_ldap.inc.php,v 1.31 2004/07/11 13:35:22 wolffc Exp $ */
+  /* $Id: class.contacts_ldap.inc.php,v 1.31.2.1 2005/08/30 20:57:03 wolffc Exp $ */
 
 	/*!
 	 @class contacts
@@ -1007,12 +1007,12 @@
 			$ldap_fields = ldap_get_entries($this->ldap, $sri);
 
 			$entry = '';
-			foreach($ldap_fields as $nul => $entry)
+			for($i=0; $i<$ldap_fields['count']; $i++)
 			{
-				$err = ldap_modify($this->ldap,$entry['dn'],array('phpgwcontactowner' => $new_owner));
+				$err = ldap_modify($this->ldap,$ldap_fields[$i]['dn'],array('phpgwcontactowner' => $new_owner));
 			}
 
-			$this->db->query("UPDATE $this->ext_table SET contact_owner='$new_owner' WHERE contact_owner=$owner",__LINE__,__FILE__);
+			$this->db->query("UPDATE $this->ext_table SET contact_owner='$new_owner' WHERE contact_owner=$old_owner",__LINE__,__FILE__);
 			return;
 		}
 
