@@ -9,7 +9,7 @@
 	*  option) any later version.                                              *
 	\**************************************************************************/
 
-	// $Id: tables_update.inc.php,v 1.95.2.9 2005/04/15 16:33:27 ralfbecker Exp $
+	// $Id: tables_update.inc.php,v 1.95.2.11 2005/08/18 12:30:49 milosch Exp $
 	// $Source: /cvsroot/egroupware/phpgwapi/setup/tables_update.inc.php,v $
 
 	/* Include older phpGroupWare update support */
@@ -1649,4 +1649,18 @@
 		$GLOBALS['setup_info']['phpgwapi']['currentver'] = '1.0.0.007';
 		return $GLOBALS['setup_info']['phpgwapi']['currentver'];
 	}
+
+	$test[] = '1.0.0.007';
+	function phpgwapi_upgrade1_0_0_007()
+	{
+		$GLOBALS['setup_info']['phpgwapi']['currentver'] = '1.0.0.008';
+		return $GLOBALS['setup_info']['phpgwapi']['currentver'];
+	}
+
+	$test[] = '1.0.0.008';
+	function phpgwapi_upgrade1_0_0_008()
+	{
+		$GLOBALS['setup_info']['phpgwapi']['currentver'] = '1.0.0.009';
+		return $GLOBALS['setup_info']['phpgwapi']['currentver'];
+	}
 ?>
