@@ -11,7 +11,7 @@
   *  option) any later version.                                              *
   \**************************************************************************/
 
-	/* $Id: class.ui.inc.php,v 1.5.2.1 2004/07/30 01:44:09 shrykedude Exp $ */
+	/* $Id: class.ui.inc.php,v 1.5.2.2 2005/03/30 15:53:08 ralfbecker Exp $ */
 
 	class ui
 	{
@@ -264,8 +264,8 @@
 			if ($_POST['submit'])
 			{
 				$newid = $this->bo->add_question();
-				$newlink = $this->adminlink('edit','answer',array('poll_id'=>$newid));
-				$this->t->set_var("message",'<a href="'.$newlink.'">'.lang("New poll has been added.  You should now add some answers for this poll").'</a>');
+				$newlink = $this->adminlink('add','answer',array('poll_id'=>$newid));
+ 				$this->t->set_var("message",'<a href="'.$newlink.'">'.lang("New poll has been added.  You should now add some answers for this poll").'</a>');
 			}
 			else
 			{
