@@ -24,7 +24,7 @@
 	* along with this program; if not, write to the Free Software       *
 	* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         *
 	\*******************************************************************/
-	/* $Id: class.uiprojects.inc.php,v 1.108.2.9 2005/02/18 13:24:46 ralfbecker Exp $ */
+	/* $Id: class.uiprojects.inc.php,v 1.108.2.11 2005/07/16 08:53:09 ralfbecker Exp $ */
 	// $Source: /cvsroot/egroupware/projects/inc/class.uiprojects.inc.php,v $
 
 	class uiprojects
@@ -413,20 +413,19 @@
 
 		function list_projects($_action=false, $_pro_main=false)
 		{
-			$action		= get_var('action',array('POST','GET'));
-			$pro_main	= get_var('pro_main',array('POST','GET'),
-				$GLOBALS['phpgw']->session->appsession('pro_main','projects'));
-
+			if (!($action = get_var('action',array('POST','GET'))))
+			{
+				$action = 'mains';
+			}
+			if ($action != 'mains')
+			{
+				$pro_main	= get_var('pro_main',array('POST','GET'),
+					$GLOBALS['phpgw']->session->appsession('pro_main','projects'));
+			}
 			if ($_GET['cat_id'])
 			{
 				$this->cat_id = $_GET['cat_id'];
 			}
-
-			if (!$action)
-			{
-				$action = 'mains';
-			}
-
 			$GLOBALS['phpgw_info']['flags']['app_header'] = lang('projects') . ': ' . ($pro_main ? lang('list jobs') : lang('list projects'));
 
 			$this->display_app_header();
@@ -968,7 +967,7 @@
 			$project_id		= get_var('project_id',array('GET','POST'));
 			$name			= get_var('name',array('POST'));
 			$values			= get_var('values',array('POST'));
-
+			
 			$link_data = array
 			(
 				'menuaction'	=> 'projects.uiprojects.list_projects',
@@ -977,7 +976,25 @@
 				'project_id'	=> $project_id,
 				'pro_parent'	=> $pro_parent
 			);
+			if($_POST['cancel'])
+			{
+				$GLOBALS['phpgw']->redirect_link('/index.php',$link_data);
+			}
+			// check if we have edit-rights for a project, if not redirect to project-list
+			if($project_id)
+			{
+				$pro = $this->boprojects->read_single_project($project_id);
 
+				if (!$this->boprojects->edit_perms(array(
+					'action'      => 'edit',
+					'coordinator' => $pro['coordinator'],
+					'main'        => $pro['main'],
+					'parent'      => $pro['parent'],
+				)))
+				{
+					$GLOBALS['phpgw']->redirect_link('/index.php',$link_data);
+				}
+			}
 			if ($_POST['save'] || $_POST['apply'])
 			{
 				//echo 'POST: SAVE';
@@ -1049,11 +1066,6 @@
 				}
 			}
 
-			if($_POST['cancel'])
-			{
-				$GLOBALS['phpgw']->redirect_link('/index.php',$link_data);
-			}
-
 			if($_POST['delete'])
 			{
 				$link_data['menuaction'] = 'projects.uiprojects.delete_project';
@@ -1747,6 +1759,18 @@
 				'action'		=> $action
 			);
 
+			// check if we have edit-rights for a project, if not redirect to project-list
+			$values = $this->boprojects->read_single_project($pa_id);
+
+			if (!$this->boprojects->edit_perms(array(
+				'action'      => 'delete',
+				'coordinator' => $values['coordinator'],
+				'main'        => $values['main'],
+				'parent'      => $values['parent'],
+			)))
+			{
+				$GLOBALS['phpgw']->redirect_link('/index.php',$link_data);
+			}
 			if ($_POST['yes'])
 			{
 				$this->boprojects->delete_project($pa_id,(isset($_POST['subs'])?True:False));
