@@ -9,7 +9,7 @@
   *  option) any later version.                                              *
   \**************************************************************************/
 
-  /* $Id: manageheader.php,v 1.72 2004/05/31 11:22:39 reinerj Exp $ */
+  /* $Id: manageheader.php,v 1.72.2.1 2005/06/16 05:36:09 ralfbecker Exp $ */
 
 	$phpgw_info = array();
 	$GLOBALS['phpgw_info']['flags'] = array(
@@ -32,9 +32,9 @@
 	$realpath = realpath('..');
 	if(!ereg('^' . $_SERVER['DOCUMENT_ROOT'],$realpath))
 	{
-		if(PHP_OS == 'Windows')
+		if(strtoupper(substr(PHP_OS, 0,3)) == 'WIN')
 		{
-			$realpath = 'Drive:\\\\Path';
+			$realpath = 'Drive:/Path/to/egroupware';
 		}
 		else
 		{
